import itebd_functions
import pylab as pl
import numpy as np
import scipy as sp
import scipy.sparse.linalg.eigen.arpack as arp
from scipy import integrate
from scipy.linalg import expm
from svd_robust import svd_gesvd
from scipy.sparse.linalg import eigs
np.set_printoptions(linewidth=2000, precision=3,threshold=4000)

######## Define the simulation parameter for bilinear-biquadratic S=1 model ###############
d = 3  # physical dimension
chi_max = 50 # maximal bond dimension, typically between a few tens and some hundreds
N = 2000 # number of iTEBD iterations to be performed
delta = 0.01 # (imaginary) time step to be used
tol= 10.**(-10) # tolerance for truncation in the bond dimension
L=2 # unit cell length for the simulation, 2 at least

g_list = [np.arctan(1./3.)] # this is the celebrated AKLT point
#g_list = np.arange(-0.4,0.5,0.1)*np.pi # this would be scanning in/outside the AFM phase
sx = np.array([[0.,1,0.],[1,0.,1],[0.,1,0.]])*np.sqrt(2.)/2.
sy = np.array([[0.,-1J,0.],[+1J,0.,-1J],[0.,+1J,0.]])*np.sqrt(2.)/2.
sz = np.array([[1.,0.,0.],[0.,0.,0.],[0.,0.,-1.]])

S_list = []
m_list = []
E0_exact_list = []
E_list = []
Eevol_list_full = []
dE_list = []
ES_list = []
xi_list = []

for g in g_list:
#    ################# Get the exact energies ######################
#    f = lambda k,g : -2*np.sqrt(1+g**2-2*g*np.cos(k))/np.pi/2.
#    E0_exact = integrate.quad(f, 0, np.pi, args=(g,))[0]
#    E0_exact_list.append(E0_exact)
    
    ############### Define the initial state: AFM state ############
    B=[];s=[]
    for i in range(2):
    #    B.append(np.zeros([d,1,1])); B[-1][0,0,0]=1
    #    s.append(np.ones([1]))        
        B.append(np.zeros([d,1,1]))
        B[-1][1+(-1)**np.mod(i,2),0,0]=1
        s.append(np.ones([1]))

    ########### Define s=1 bil-biq Hamiltonian and get U ################
    H=[]; U=[];
    scal = np.kron(sx,sx) + np.kron(sy,sy) + np.kron(sz,sz)
    Htmp = np.cos(g)*scal+np.sin(g)*np.dot(scal,scal)
    for i in range(L):
        H.append(Htmp)
        U.append(np.reshape(expm(-delta*Htmp),(d,d,d,d)))
        
    Eevol_list = []
    ################# Do the iTEBD run ############################
    for step in range(1, N):
        itebd_functions.update(B,s,U,chi_max,tol)
        Etmp=np.mean(itebd_functions.bond_expectation_value(B,s,np.reshape(H[0],(d,d,d,d))))
        Eevol_list.append(Etmp)
    
    ################# Measure stuff  ##############################
    S=np.mean(itebd_functions.entanglement_entropy(s,tol))
    S_list.append(S)
    ES=itebd_functions.entanglement_spectrum(s,tol)
    ES_list.append(ES)
    
    m=np.mean(np.abs(itebd_functions.site_expectation_value(B,s,sz)))
    m_list.append(m)
    
    E=np.mean(itebd_functions.bond_expectation_value(B,s,np.reshape(H[0],(d,d,d,d))))
    E_list.append(E)
    
    xi=itebd_functions.correlation_length(B,s)
    xi_list.append(xi)
    
    Eevol_list_full.append(Eevol_list)
#    dE_list.append(np.abs((E-E0_exact)/E0_exact))
    #print "chi=%3.0f"%chi[0]," S=%.4f"%S," m=%.4f"%m,"E=%.8f"%E, "E_exact=%.8f"%E0_exact
    print "g=%g \\pi done" %(g/np.pi)

# for the very specific exactly solvable AKLT point, the energy is easy to be computed 
ii=5 # index within g_list, i.e., which coupling are we looking at
g=g_list[ii]

fig2 = pl.figure(figsize=[8,4])
pl.semilogy(np.abs(Eevol_list_full[ii]-E_list[ii]),'o')
pl.ylabel('$\\Delta E [g= %g \\pi]$' % (g/np.pi))
pl.xlabel('$steps$')
pl.show();


# for the very specific exactly solvable AKLT point, the energy is easy to be computed 
#ii=0 # index within g_list, i.e., which coupling are we looking at
#g=g_list[ii]
#E0_exact=-np.cos(g)*2./3.
#E_list[ii]-E0_exact


ii=5 # index within g_list, i.e., which coupling are we looking at
g=g_list[ii]
fig2 = pl.figure(figsize=[8,4])
pl.semilogy(ES_list[ii][1],'o')
pl.ylabel('$ES [g= %g \\pi]$' % (g/np.pi))
pl.xlabel('$\\alpha$')
pl.show();