import itebd_functions
import pylab as pl
import numpy as np
import scipy as sp
import scipy.sparse.linalg.eigen.arpack as arp
from scipy import integrate
from scipy.linalg import expm
from svd_robust import svd_gesvd
from scipy.sparse.linalg import eigs
np.set_printoptions(linewidth=2000, precision=3,threshold=4000)


""" Conventions:
B[i,a,b] has axes (physical, left virtual, right virtual),
W[a,b,i,j] has axes (virtual left, virtual right, physical out, physical in)
S[i] are schmidt values between sites (i-1, i+1),
H_bond[i] is the bond hamiltonian between (i,i+1) with (only physical)
axes (out left, out right, in left, in right)"""


def init_cdw_mps(L):
    """ Returns alternating charge on physical sites """
    d = 2
    B = []
    s = []
    for i in range(L):
        B.append(np.zeros([2,1,1])); B[-1][i%L,0,0]=1
        s.append(np.ones([1]))
    s.append(np.ones([1]))
    return B,s

def init_rice_mele_U_bond(t,u,V,delta):
    """ Returns bond hamiltonian and bond time-evolution"""
    bp = np.array([[0.,0.],[1.,0.]])
    bm = bp.T
    n = np.dot(bp,bm)
    d = 2

    U_bond = []
    U_bond_real = []
    H_bond = []
    for i in range(2):
        H = t[i]*(np.kron(bp,bm) + np.kron(bm,bp)) + u[i]*np.kron(n,np.eye(2)) + V*np.kron(n,n)
        H_bond.append(np.reshape(H,(d,d,d,d)))
        U_bond.append(np.reshape(expm(-delta*H),(d,d,d,d)))

    return U_bond,H_bond

def tebd_rice_mele(t,u,V,N,B,s,chi,tol):
# service subroutine to perform the cooling down (imaginary time evolution) at a given parameter set
    for delta in [0.1,0.01,0.001]: #0
        U_bond,H_bond = init_rice_mele_U_bond(t,u,V,delta)
        for i in range(int(N)):
            itebd_functions.update(B,s,U_bond,chi,tol)
                
    return B,s   


def n_right(B,s,eps,Ns):
# we follow here the procedure of Pollmann et al., https://arxiv.org/pdf/1211.3733.pdf, page 8
# set eps and Ns for the regularization from the outside, e.g., eps=0.1 & Ns=201
    
    L=len(B)
    chi=B[1].shape[2]

    s0 = np.eye(2)
    sz = np.array([[0.5, 0.0], [ 0.0, -0.5]])

# Rp is initialized as a chi x chi identity, tensor a boundary vector [0,1] that acts at MPO level
# (result has axes (a,a',m') = (chi, chi, 2) = (virtual up, virtual down, MPO virtual left) )
# and then we will iteratively redefine it to become the (weighted) sum of sz 
# on Ns sites starting from the right, i.e., \sum_(i_s) (1-exp(-eps*i_s)) sz(i_s)
    Rp = np.tensordot(np.eye(chi),np.array([0,1]),axes=0) # a a' m
    for i_s in range(1,Ns):
        M = np.zeros((2,2,2,2))
# write the MPO  M representing the mentioned (weighted) sum of sz,
# recall the convention (m,m',i,i') = (virtual left, virtual right, physical out, physical in)

################################################################
#  YOUR CONTRIBUTION NEEDED !  ##  TWO LINES  #
################################################################

# then contract it with Rp on the index m',
# then the result with B and conj(B) on the indexes (i,a) and (i',a'), respectively
# in order to get again a three axes object of the same starting form -- see blackboard

################################################################
#  YOUR CONTRIBUTION NEEDED !  ##  THREE LINES  #
################################################################
    
# here, we close the MPO on the left side with a boundary vector [[1],[0]], 
# and then, following the recipe of Pollmann et al., we take away the spurious boundary contribution
# in order to get the amount of extra charge present in the right hand side of our system
    nrtmp=np.diag(Rp[:,:,0])
    nr=np.real(np.sum((s[0]**2)*(nrtmp-nrtmp[0]))) #%1 # you may count modulo 1 to get smoother plots, but that's an abuse...
#    print "nrtmp", [np.sum((s[0]**2)*nrtmp),nrtmp[0],np.sum((s[0]**2))] 
#    return np.diag(Rp[:,:,0])
    return nr
